{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VGA320x240 4 color Bitmap Engine
//
// Author: Kwabena W. Agyeman
// Updated: 7/15/2010
// Designed For: P8X32A
// Version: 1.1
//
// Copyright (c) 2010 Kwabena W. Agyeman
// See end of file for terms of use.
//
// Update History:
//
// v1.0 - Original release - 9/28/2009.
// v1.1 - Merged and rewrote code and added more features - 7/15/2010.
//
// For each included copy of this object only one spin interpreter should access it at a time.
//
// Nyamekye,
// Modifications James Moxham April 2011 so main screen buffer is in the 'main' program
// and able to be turned into a sd loadable cogject
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}
CON

  _clkfreq = 80_000_000                                              ' 5Mhz Crystal
  _clkmode = xtal1 + pll16x                                          ' x 16

PUB Main
    coginit(1,@entry,0)                                           ' cog 1, cogstart, dummy value

DAT

' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'                       BMP Driver
' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                        org     0

' //////////////////////Initialization/////////////////////////////////////////////////////////////////////////////////////////

entry                   mov     i,                     par                       ' video buffer                
                        rdlong  screenorg,             i
                        add     i,#4 
                        rdlong  displayIndicatorAddress,i                       ' read in ongs from video buffer
                        add     i,#4
                        rdlong  syncIndicatorAddress,i
                        add     i,#4
                        rdlong  directionstate,i
                        add     i,#4
                        rdlong  videostate,i
                        add     i,#4
                        rdlong  frequencystate,i
                        add     i,#4 
                        rdlong  horizontalScaling,i
                        add     i,#4
                        rdlong  verticalScaling,i
                        add     i,#4  
                        rdlong  horizontalPixels,i
                        add     i,#4 
                        rdlong  verticalPixels,i
                        add     i,#4
                        rdlong  visibleScale,i
                        add     i,#4 
                        rdlong  invisibleScale,i
                        add     i,#4 
                        rdlong  horizontalLongs,i
                        add     i,#4
                        rdlong  horizontalLoops,i
                        add     i,#4
                        rdlong  pixelcolorsaddress,i

                        mov     vcfg,                 videoState                    ' Setup video hardware.
                        mov     frqa,                 frequencyState                '
                        movi    ctra,                 #%0_00001_101                 '

' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'                       Active Video
' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

loop                    mov     buffer,               screenorg                     ' get the screen origin 
                        mov     tilesCounter,         verticalPixels                '

tilesDisplay            mov     tileCounter,          verticalScaling               ' Set/Reset tile fill counter.

tileDisplay             mov     vscl,                 visibleScale                  ' Set/Reset the video scale.
                        mov     counter,              horizontalLongs               '

' //////////////////////Visible Video//////////////////////////////////////////////////////////////////////////////////////////

videoLoop               rdlong  screenPixels,         buffer                        ' Download new pixels.
                        add     buffer,               #4                            '

                        waitvid screenColors,         screenPixels                  ' Update display scanline.

                        djnz    counter,              #videoLoop                    ' Repeat.

' //////////////////////Invisible Video////////////////////////////////////////////////////////////////////////////////////////

                        mov     vscl,                 invisibleScale                ' Set/Reset the video scale.

                        waitvid HSyncColors,          syncPixels                    ' Horizontal Sync.

' //////////////////////Repeat/////////////////////////////////////////////////////////////////////////////////////////////////

                        sub     buffer,               horizontalLoops               ' Repeat.
                        djnz    tileCounter,          #tileDisplay                  '

                        add     buffer,               horizontalLoops               ' Repeat.
                        djnz    tilesCounter,         #tilesDisplay                 '

' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'                       Inactive Video
' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                        rdlong  screenColors,         pixelColorsAddress            ' Get new screen colors.
                        or      screenColors,         HVSyncColors                  '

' //////////////////////Update Indicator///////////////////////////////////////////////////////////////////////////////////////

                        add     refreshCounter,       #1                            ' Update sync indicator.
                        wrbyte  refreshCounter,       syncIndicatorAddress          '

' //////////////////////Front Porch////////////////////////////////////////////////////////////////////////////////////////////

                        mov     counter,              #11                           ' Set loop counter.

frontPorch              mov     vscl,                 blankPixels                   ' Invisible lines.
                        waitvid HSyncColors,          #0                            '

                        mov     vscl,                 invisibleScale                ' Horizontal Sync.
                        waitvid HSyncColors,          syncPixels                    '

                        djnz    counter,              #frontPorch                   ' Repeat # times.

' //////////////////////Vertical Sync//////////////////////////////////////////////////////////////////////////////////////////

                        mov     counter,              #(2 + 2)                      ' Set loop counter.

verticalSync            mov     vscl,                 blankPixels                   ' Invisible lines.
                        waitvid VSyncColors,          #0                            '

                        mov     vscl,                 invisibleScale                ' Vertical Sync.
                        waitvid VSyncColors,          syncPixels                    '

                        djnz    counter,              #verticalSync                 ' Repeat # times.

' //////////////////////Back Porch/////////////////////////////////////////////////////////////////////////////////////////////

                        mov     counter,              #31                           ' Set loop counter.

backPorch               mov     vscl,                 blankPixels                   ' Invisible lines.
                        waitvid HSyncColors,          #0                            '

                        mov     vscl,                 invisibleScale                ' Horizontal Sync.
                        waitvid HSyncColors,          syncPixels                    '

                        djnz    counter,              #backPorch                    ' Repeat # times.

' //////////////////////Update Display Settings////////////////////////////////////////////////////////////////////////////////

                        rdbyte  buffer,               displayIndicatorAddress wz    ' Update display settings.
                        muxnz   dira,                 directionState                '

' //////////////////////Loop///////////////////////////////////////////////////////////////////////////////////////////////////

                        jmp     #loop                                               ' Loop.

' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'                       Data
' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

blankPixels             long    640                                                 ' Blank scanline pixel length.
syncPixels              long    $00_00_3F_FC                                        ' F-porch, h-sync, and b-porch.
HSyncColors             long    $01_03_01_03                                        ' Horizontal sync color mask.
VSyncColors             long    $00_02_00_02                                        ' Vertical sync color mask.
HVSyncColors            long    $03_03_03_03                                        ' Horizontal and vertical sync colors.

' //////////////////////Configuration Settings/////////////////////////////////////////////////////////////////////////////////

directionState          long    0
videoState              long    0
frequencyState          long    0
horizontalScaling       long    0
verticalScaling         long    0
horizontalPixels        long    0
verticalPixels          long    0
visibleScale            long    0
invisibleScale          long    0
horizontalLongs         long    0
horizontalLoops         long    0
screenorg               long    0               ' added instead of reading it from par each loop

' //////////////////////Addresses//////////////////////////////////////////////////////////////////////////////////////////////

pixelColorsAddress      long    0
displayIndicatorAddress long    0
syncIndicatorAddress    long    0

' //////////////////////Run Time Variables/////////////////////////////////////////////////////////////////////////////////////

counter                 res     1
buffer                  res     1

tileCounter             res     1
tilesCounter            res     1

screenPixels            res     1
screenColors            res     1

refreshCounter          res     1
displayCounter          res     1
i                       res     1               ' general purpose variable


' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                        fit     496

' /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}